-- Copyright (C) 2009 Papavasileiou Dimitris                             
--                                                                      
-- This program is free software: you can redistribute it and/or modify 
-- it under the terms of the GNU General Public License as published by 
-- the Free Software Foundation, either version 3 of the License, or    
-- (at your option) any later version.                                  
--                                                                      
-- This program is distributed in the hope that it will be useful,      
-- but WITHOUT ANY WARRANTY; without even the implied warranty of       
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
-- GNU General Public License for more details.                         
--                                                                      
-- You should have received a copy of the GNU General Public License    
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.

return function (values)
   local meta, self, oldindex, oldnewindex

   self = frames.transform {
      pressed = false,
      momentary = false,

      case = (options.toon and toon.cel or shading.cook) {
      	 color = options.toon and math.scale({1, 1, 1}, 0.2),
      	 thickness = options.toon and 2,


      	 diffuse = not options.toon and math.scale ({1, 0.9, 1}, 0.15),
      	 specular = not options.toon and math.scale ({0.12, 0.17, 0.19}, 0.7),
      	 parameter = not options.toon and {1.3, 0.15},
	 
      	 mesh = resources.static "aviation/meshes/buttoncase.lc" {},
      },

      axis = frames.transform {
	 button = switches.button {
	    surface = (options.toon and toon.cel or shading.cook) {
	       color = options.toon and math.scale({0.8441, 0.0111, 0.0119}, 0.7),
	       thickness = options.toon and 2,

	       diffuse = not options.toon and math.scale({0.8441, 0.0111, 0.0119}, 0.5),
	       specular = not options.toon and math.scale ({0.12, 0.17, 0.19}, 1), 
	       parameter = not options.toon and {0.95, 0.2},

	       mesh = resources.static "aviation/meshes/button.lc" {},
	    },
	    
	    highlight = widgets.highlight {
	       mesh = resources.static "aviation/meshes/button.lc" {},

	       color = {0.7, 0.6, 0.1},
	       width = 5,
	       thickness = 1.5,
	       radius = 0.03,
	       angle = 60,
	       
	       text = string.format([[<span font="Sans 10" color="Khaki">Button</span>]])
	    },

	    selected = frames.event {
	       link = function (self)
			 self.parent.highlight.speed = 8
		      end,

	       unlink = function (self)
			   self.parent.highlight.speed = -4
			end,

	       buttonpress = function (self, button, x, y)
				if self.ancestry[3].momentary then
				   self.ancestry[3].pressed = true
				else
				   self.ancestry[3].pressed = not self.ancestry[3].pressed
				end
			     end,
	       
	       buttonrelease = function (self, button, x, y)
				if self.ancestry[3].momentary then
				   self.ancestry[3].pressed = false
				end
			       end,
	    },
	 },
      },
   }

   meta = getmetatable (self)
   meta.label = "Button"
   meta.press = nil

   oldindex = meta.__index
   oldnewindex = meta.__newindex

   meta.__index = function (self, key)
		     local meta = getmetatable(self)

		     if key == "label" then
			return meta.label
		     elseif key == "press" then
			return meta.press
		     elseif key == "pressed" then
			return oldindex (self, key)
		     else
			return oldindex (self, key)
		     end
		  end

   meta.__newindex = function (self, key, value)
      local meta = getmetatable(self)

      if key == "label" then
	 meta.label = value

	 self.axis.button.highlight.text = string.format([[<span font="Sans 10" color="Khaki">%s</span>]], value)
      elseif key == "press" then
	 meta.press = value
      elseif key == "pressed" then
	 oldnewindex(self, key, value)

	 if self.pressed then
	    self.axis.position = {0, 0, -0.005}
	 else
	    self.axis.position = {0, 0, 0}
	 end

	 if self.press then
	    self.press (self)
	 end
      else
	 oldnewindex(self, key, value)
      end
   end

   meta.__tostring = function(self)
			 return "button"
		      end

   for key, value in pairs(values) do
      self[key] = value
   end

   return self
end
